"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.downloadThemeLiquidDocs = exports.exists = exports.schemaUrl = exports.schemaPath = exports.resourceUrl = exports.resourcePath = exports.downloadResource = exports.downloadSchema = exports.Manifests = exports.Resources = exports.ThemeLiquidDocsSchemaRoot = exports.ThemeLiquidDocsRoot = exports.root = void 0;
const env_paths_1 = __importDefault(require("env-paths"));
const node_fetch_1 = __importDefault(require("node-fetch"));
const promises_1 = __importDefault(require("node:fs/promises"));
const node_path_1 = __importDefault(require("node:path"));
const paths = (0, env_paths_1.default)('theme-liquid-docs');
exports.root = paths.cache;
exports.ThemeLiquidDocsRoot = 'https://raw.githubusercontent.com/Shopify/theme-liquid-docs/main';
exports.ThemeLiquidDocsSchemaRoot = `${exports.ThemeLiquidDocsRoot}/schemas`;
exports.Resources = [
    'filters',
    'objects',
    'tags',
    'shopify_system_translations',
    'manifest_theme',
    'manifest_theme_app_extension',
];
exports.Manifests = {
    app: 'manifest_theme_app_extension',
    theme: 'manifest_theme',
};
const THEME_LIQUID_DOCS = {
    filters: 'data/filters.json',
    objects: 'data/objects.json',
    tags: 'data/tags.json',
    latest: 'data/latest.json',
    shopify_system_translations: 'data/shopify_system_translations.json',
    manifest_theme: 'schemas/manifest_theme.json',
    manifest_theme_app_extension: 'schemas/manifest_theme_app_extension.json',
};
async function downloadSchema(relativeUri, destination = exports.root) {
    const remotePath = schemaUrl(relativeUri);
    const localPath = schemaPath(relativeUri, destination);
    const res = await (0, node_fetch_1.default)(remotePath);
    const text = await res.text();
    promises_1.default.writeFile(localPath, text, 'utf8');
    return text;
}
exports.downloadSchema = downloadSchema;
async function downloadResource(resource, destination = exports.root) {
    const remotePath = resourceUrl(resource);
    const localPath = resourcePath(resource, destination);
    const res = await (0, node_fetch_1.default)(remotePath);
    const text = await res.text();
    promises_1.default.writeFile(localPath, text, 'utf8');
    return text;
}
exports.downloadResource = downloadResource;
function resourcePath(resource, destination = exports.root) {
    return node_path_1.default.join(destination, `${resource}.json`);
}
exports.resourcePath = resourcePath;
function resourceUrl(resource) {
    const relativePath = THEME_LIQUID_DOCS[resource];
    return `${exports.ThemeLiquidDocsRoot}/${relativePath}`;
}
exports.resourceUrl = resourceUrl;
function schemaPath(relativeUri, destination = exports.root) {
    return node_path_1.default.resolve(destination, node_path_1.default.basename(relativeUri));
}
exports.schemaPath = schemaPath;
function schemaUrl(relativeUri) {
    return `${exports.ThemeLiquidDocsSchemaRoot}/${relativeUri}`;
}
exports.schemaUrl = schemaUrl;
async function exists(path) {
    try {
        await promises_1.default.stat(path);
        return true;
    }
    catch (e) {
        return false;
    }
}
exports.exists = exists;
async function downloadThemeLiquidDocs(destination = exports.root) {
    if (!(await exists(destination))) {
        await promises_1.default.mkdir(destination);
    }
    const resources = ['latest'].concat(exports.Resources);
    const resourceContents = await Promise.all(resources.map((file) => {
        return downloadResource(file, destination);
    }));
    const manifests = Object.values(exports.Manifests)
        .map((resource) => resources.indexOf(resource))
        .map((index) => resourceContents[index])
        .map((manifest) => JSON.parse(manifest));
    const relativeUris = manifests.flatMap((manifest) => manifest.schemas.map((schema) => schema.uri));
    await Promise.all(unique(relativeUris).map((uri) => downloadSchema(uri, destination)));
}
exports.downloadThemeLiquidDocs = downloadThemeLiquidDocs;
function unique(array) {
    return [...new Set(array).values()];
}
//# sourceMappingURL=themeLiquidDocsDownloader.js.map